/*global require module window*/
(function(){"use strict";
	var util = require('./util'),
	    Class = require('./Class');
	var Guide = module.exports = Class.create('Guide', function (obj, proto) {
		obj.initGuide = function (routes) {
			this.routes = {};
			return this.def(routes);
		};
		obj.def = function (obj, route) {
			if (!obj) return this;
			route = route || this.routes;
			util.enumerate(obj, function (value, path) {
				this.set(path, value, route);
			}, this);
			return this;
		};
		obj.set = function (path, value, route) {
				if (route) {
				if (path === '' && util.isFunction(value)) {
					route[''] = value;
					return this;
				}
				if (path === '_name_' && typeof route._name_ === 'string' && typeof value === 'string') {
					route._name_ = value;
					return this;
				}
			}
			route = this.add(path, route);
			if (!route) return this;
			if (util.isFunction(value)) {
				route[''] = value;
			} else if (util.isObjectLike(value)) {
				this.def(value, route);
			}
			return this;
		};
		obj.add = function (path, route) {
			route = route || this.routes;
			util.enumerate(this.parse(path), function (segment) {
				if (!route) return;
				if (segment.regex) {
					route = this.addExpr(route, segment.text);
					route._name_ = '';
				} else {
					route = this.find(route, segment.text, true);
				}
			}, this);
			return route;
		};
		obj.get = function (path, route, withoutInfo) {
			route = route || this.routes;
			var done, info = {
				router: this,
				from: route,
				path: this.parse(path),
				expr: []
			};
			util.enumerate(info.path, function (segment) {
				if (done || !route || segment.text === '') return;
				var next = this.find(route, segment.text) ||
					this.getExpr(route, segment.text, info);
				if (next) {
					route = next;
				} else {
					done = true;
				}
			}, this);
			if (route && !withoutInfo) {
				route._info_ = info;
			}
			return route;
		};
		obj.find = function (route, text, add) {
			if (!route) return null;
			route = route[text] || (add && (route[text] = {}) || null);
			return route;
		};
		obj.send = function (route, callback, bind) {
			if (typeof route === 'string') {
				route = this.get(route);
			}
			var _info = route && route._info_,
			    info = {};
			if (route) delete route._info_;
			callback = util.isFunction(callback) ? callback : function (proc, info) {
				proc.call(this, info);
			};
			bind = bind || this;
			if (route && util.isFunction(route[''])) {
				if (_info) util.enumerate(_info.expr, function (subinfo) {
					info[subinfo.name] = subinfo.text;
				});
				callback.call(bind, route[''], info);
			} else {
				callback.call(bind);
			}
			return this;
		};
		obj.parse = function (path) {
			if (typeof path !== 'string') return [];
			var text = '',
			    escape = false,
			    regex = false,
			    paths = [];
			util.enumerate(path.split(''), function (c) {
				switch (c) {
					case '#':
						escape = !escape;
						if (!escape) text += c;
						break;
					case '/':
						if (escape) text += c;
						else {
							paths.push({text: text, regex: regex});
							text = '';
							regex = false;
						}
						escape = false;
						break;
					case '^':
						if (!escape && text === '') {
							regex = true;
						}
						text += c;
						escape = false;
						break;
					default: 
						text += c;
						escape = false;
				}
			});
			if (text && text.length) {
				paths.push({text: text, regex: regex});
			}
			return paths;
		};
		obj.addExpr = function (route, regex) {
			if (!route) return null;
			regex = new RegExp(regex);
			var expr = route._expr_ || (route._expr_ = {_list_: []});
			var look = util.keyOf(expr.list, regex, function (val, targ) {
				return val && val.regex && val.regex.source === targ.source;
			});
			if (look > -1) {
				route = expr.list[look];
			} else {
				route = {regex: regex, route: {}};
				expr._list_.push(route);
			}
			return route.route;
		};
		obj.getExpr = function (route, text, info) {
			if (!route || !route._expr_ || !route._expr_._list_) {
				return null;
			}
			var expr = route._expr_,
			    name,
			    match,
			    index;
			util.enumerate(expr._list_, function (subroute, ind) {
				if (match) return;
				if (subroute.regex.test(text)) {
					match = subroute.route;
					index = ind;
					name = match._name_;
				}
			});
			info.expr.push({
				expr: expr,
				name: name,
				text: text,
				index: index,
				match: match
			});
			return match;
		};
	});
}).call(this);